<?php

use Tygh\Registry;
use Tygh\Settings;
use Tygh\Http;

use Tygh\Addons\RusTaxes\TaxType;
use Tygh\Addons\SwBifitOnline\Logs;
use Tygh\Addons\SwBifitOnline\Client;
use Tygh\Addons\SwBifitOnline\Exception;
use Tygh\Addons\SwBifitOnline\CheckContent;
use Tygh\Addons\SwBifitOnline\Processing;

if (!defined('BOOTSTRAP')) { die('Access denied'); }


/**
 * Gets extra settings.
 *
 * @return array
 */
function fn_sw_bifit_online_get_settings()
{
    if (!Registry::isExist('sw_bifit_online.extra_settings')) {
        $settings = json_decode(Registry::get('addons.sw_bifit_online.extra'), true);

        if (!is_array($settings)) {
            $settings = array();
        }

        Registry::set('sw_bifit_online.extra_settings', $settings, false);
    }

    return (array) Registry::get('sw_bifit_online.extra_settings');
}


/**
 * Add check status.
 *
 * @return id
 */
function fn_sw_bifit_online_add_check_status($res, $order_id)
{
    $status = false;
   
    if(isset($res['response'])){
       
        if(isset($res['response']['error'])){
            $status = false;
        }else{
            $status = true;
        }
    }
    
    $data = array(
        'order_id'   => $order_id,
        'status'     => $status,
        'timestamp'  => TIME
    );
    db_query('REPLACE INTO ?:bifit_online_order_data ?e', $data);
   
   return;
}

/**
 * Add receipts data.
    id    	  			- id AI
    check_id 			- check_id
    object_id 			- order_id
    type 	  			- ReceiptType 
    taxSystem 			- taxSystem
    cashier_name	    - cashier data
    cashier_address     - cashier data
    timestamp			- timestamp
    created_timestamp   - created
    updated_timestamp   - changed
    payments			- payments
    status				- status
    response  			- seiralize short data
    full_data 			- full data response
 *
 * @return id
 */
function fn_sw_bifit_online_add_document_data($res, $order_id = 0)
{
    
    if(empty($res) || !isset($res['id'])){
       return; 
    }
     
    $data = array(
        'check_id' => $res['id'],
        'object_id' => $order_id,
        'type' => $res['request']['type'],
        'status' => $res['status'],
        'timestamp' => TIME,
        'created_timestamp' => $res['created'],
        'updated_timestamp' => $res['changed'],
        'taxSystem' => $res['request']['taxSystem'],
        'cashier_name' => $res['request']['cashier']['name'],
        'cashier_address' => $res['request']['cashier']['inn'],
        'client_name' => $res['request']['client']['name'],
        'client_address' => $res['request']['client']['address'],
        'payments' => json_encode($res['request']['payments']),
        'response' => json_encode($res['response'], JSON_UNESCAPED_UNICODE),
        'full_data' => json_encode($res, JSON_UNESCAPED_UNICODE),
    );
    
    $id = db_get_field("SELECT id FROM ?:bifit_online_receipts WHERE check_id = ?i", $res['id']);
    
    if(empty($id)){
        db_query('INSERT INTO ?:bifit_online_receipts ?e', $data);
    }else{
        unset($data['object_id']);
        db_query('UPDATE ?:bifit_online_receipts SET ?u WHERE id = ?i', $data, $id);
    }
   
   return;
}

function fn_sw_bifit_online_check_register_payment($payment_id){
    
    $check = db_get_field("SELECT bifit_payment_type FROM ?:payments WHERE payment_id = ?i", $payment_id);
    
    return $check;
}


/**
 * Gets external payments.
 *
 * @return array
 */
function fn_sw_bifit_online_get_external_payments()
{
    return fn_get_schema('sw_bifit_online', 'payments');
}

/**
 * Check 
 *
 * @return array
 */
function fn_sw_bifit_online__update_receipts_statuses()
{
    $need_update_statuses = array(
        'QUEUE', 'REGISTRATION'   
    );
    
    $receipts_ids = db_get_array("SELECT check_id, object_id FROM ?:bifit_online_receipts WHERE status IN (?a)", $need_update_statuses);
    
    if(!empty($receipts_ids)){
        
        $client = new Client();
        $token = $client->getToken();
        
        foreach($receipts_ids as $receipt_data){
            
            $order_info = fn_get_order_info($receipt_data['object_id']);
            if(!empty($order_info)){
                $res = $client->get_document($receipt_data['check_id'], $token, $order_info);
                fn_sw_bifit_online_add_document_data($res);
            }

        }
    }
    
    return;
}

/*
* get link on check
* @param json array()
* return $url (string)
*/

function fn_sw_bifit_online__get_receipt_link($data)
{
    $url = '';
    
    if(!empty($data)){
        $data = json_decode($data, true);
        if(isset($data['idempotencyKey'])){
            $url = 'https://fp.bifit.com/processing-api/receipts/'.$data['idempotencyKey'];
        }
    }
    
    return $url;
}



/**
 * Функция проверяет правильность инн
 *
 * @param string $inn
 * @return bool
 */
function fn_sw_bifit_online_is_valid_inn( $inn )
{
    if ( preg_match('/\D/', $inn) ) return false;
    
    $inn = (string) $inn;
    $len = strlen($inn);
    
    if($len < 10){
       fn_set_notification("W",__("warning"),__('sw_bifit_online.error_inn_10')); 
    }elseif($len > 12){
        fn_set_notification("W",__("warning"),__('sw_bifit_online.error_inn_12')); 
    }elseif($len === 11){
        fn_set_notification("W",__("warning"),__('sw_bifit_online.error_inn_11')); 
    }
    
    if ( $len === 10 )
    {
        return $inn[9] === (string) (((
            2*$inn[0] + 4*$inn[1] + 10*$inn[2] + 
            3*$inn[3] + 5*$inn[4] +  9*$inn[5] + 
            4*$inn[6] + 6*$inn[7] +  8*$inn[8]
        ) % 11) % 10);
    }
    elseif ( $len === 12 )
    {
        $num10 = (string) (((
             7*$inn[0] + 2*$inn[1] + 4*$inn[2] +
            10*$inn[3] + 3*$inn[4] + 5*$inn[5] + 
             9*$inn[6] + 4*$inn[7] + 6*$inn[8] +
             8*$inn[9]
        ) % 11) % 10);
        
        $num11 = (string) (((
            3*$inn[0] +  7*$inn[1] + 2*$inn[2] +
            4*$inn[3] + 10*$inn[4] + 3*$inn[5] +
            5*$inn[6] +  9*$inn[7] + 4*$inn[8] +
            6*$inn[9] +  8*$inn[10]
        ) % 11) % 10);
        
        return $inn[11] === $num11 && $inn[10] === $num10;
    }
    
    return false;
}

/**
 * Hook handler: after order status changed.
 *
 * @param string $status_to     Order status to
 * @param string $status_from   Order status from
 * @param array  $order_info    Order data
 */
function fn_sw_bifit_online_change_order_status($status_to, $status_from, $order_info)
{
    if ($order_info['is_parent_order'] === 'Y') {
        return;
    }
    
    $order_id = isset($order_info['order_id']) ? $order_info['order_id'] : 0;
    
    $addon_settings = Registry::get('addons.sw_bifit_online');

    $statuses_paid = $addon_settings['statuses_paid'];
    $statuses_refund = $addon_settings['statuses_refund'];
    $payment_id = isset($order_info['payment_id']) ? $order_info['payment_id'] : 0;
    $register_payment_id = fn_sw_bifit_online_check_register_payment($payment_id);
    
   
    
    if ($register_payment_id === '') {
        return;
    }
    
    $is_paid = false;
    if(isset($statuses_paid[$status_to]) && $statuses_paid[$status_to] == 'Y'){
        $is_paid = true;
    }
    
    $is_refund = false;
    if(isset($statuses_refund[$status_to]) && $statuses_refund[$status_to] == 'Y'){
        $is_refund = true;
    }
    
    if($is_paid == true || $is_refund == true){
        $client = new Client();
        
        //Авторизация
        $token = $client->getToken();
        
        //Формируем контент чека
        $CheckContent = new CheckContent($token, $addon_settings);
        $check_data = $CheckContent->getCheckContent($order_info, $is_refund);
        
        //Отправка чека
        $processing = new Processing($token, $addon_settings);
        $check_id = $processing->send($check_data, $order_info);
        
        if(!empty($check_id)){
            //Формируем документ
            $res = $client->get_document($check_id, $token, $order_info);
            
            fn_sw_bifit_online_add_check_status($res, $order_id);
            fn_sw_bifit_online_add_document_data($res, $order_id);
        }
    }
}


/**
 * System settings
 */
function fn_settings_actions_addons_sw_bifit_online(&$new_status, $old_status, $on_install){
    if($new_status == 'A'){
        // $new_status = fn_sw_bifit_online_update_addon_status_ckeckin($on_install);
    }

}

function fn_sw_bifit_online_update_addon_status_ckeckin($on_install, $update = false){$_='sw_bifit_online';$addon_data = fn_get_addon_settings_values($_);$d = $_SERVER['HTTP_HOST'];$url = base64_decode('aHR0cHM6Ly9zd2VldGNhcnQucnU=');$data = array(base64_decode('ZGlzcGF0Y2g==') => base64_decode('bGljZW5zZV9jaGVja18yLmdldA='.'='),base64_decode('aWRfYWRkb24'.'=') => $_,base64_decode('ZG9tYWlu') => $d,'vers' => fn_get_addon_version($_),'lang_code' => DESCR_SL);if($on_install == true){$data['timestamp'] = TIME;}if (fn_allowed_for('ULTIMATE')){$data['ult'] = true;}if (fn_allowed_for('MULTIVENDOR')) {$data['mv'] = true;}$data_query = Http::get($url, $data);$data_query = json_decode($data_query, true);if(!empty($data_query)){if(isset($data_query['error']) && $data_query['error'] == true){fn_set_notification("W",__("warning"),$data_query['message']);}$status = $data_query['status'];}if($update == true && $status == 'D'){fn_update_addon_status($_, $status);}return $status;}